//############################################
//
// for mega / haas matrix keyboard
// if shift flag uses shift array
// if 0 in the standard layout will look in 
// the alternate layout
//
// uses modified keypad library 
// modified for ulong x ulong matrix and
// modified for int based USB scancode arrays (rather then char)
//
// requires 16u2 bootloader for HID device
// http://mitchtech.net/arduino-usb-hid-keyboard/
//
//#############################################

#include <Keypad_int.h>

#define KEY_LEFT_CTRL   0x01
#define KEY_LEFT_SHIFT  0x02
#define KEY_LEFT_ALT    0x04
#define KEY_LEFT_GUI    0x08
#define KEY_RIGHT_CTRL  0x10
#define KEY_RIGHT_SHIFT 0x20

// These modifier flags will be
// right shifted by 8 to become the
// same as the values above 
#define _C 256 // left control flag
#define _S 512 // right shift flag

// slightly easier to read then plain numbers
#define K_NOKEY 0
#define K_A 4
#define K_B 5
#define K_C 6
#define K_D 7
#define K_E 8

#define K_F 9
#define K_G 10
#define K_H 11
#define K_I 12
#define K_J 13

#define K_K 14
#define K_L 15
#define K_M 16
#define K_N 17
#define K_O 18

#define K_P 19
#define K_Q 20
#define K_R 21
#define K_S 22
#define K_T 23

#define K_U 24
#define K_V 25
#define K_W 26
#define K_X 27
#define K_Y 28

#define K_Z 29
#define K_1 30
#define K_2 31
#define K_3 32
#define K_4 33

#define K_5 34
#define K_6 35
#define K_7 36
#define K_8 37
#define K_9 38

#define K_0 39
#define K_ENTER 40
#define K_ESC 41
#define K_BSPC 42
#define K_TAB 43

#define K_SPACE 44
#define K_MNS 45
#define K_EQL 46
#define K_LBRKT 47
#define K_RBRKT 48

#define K_BSLASH 49
#define K_50 50
#define K_SCOLON 51
#define K_SQUOTE 52
#define K_GRAVE 53

#define K_COMMA 54
#define K_PERIOD 55
#define K_FSLASH 56
#define K_CAPSL 57
#define K_F1 58

#define K_F2 59
#define K_F3 60
#define K_F4 61
#define K_F5 62
#define K_F6 63

#define K_F7 64
#define K_F8 65
#define K_F9 66
#define K_F10 67
#define K_F11 68

#define K_F12 69
#define K_PRTSCRN 70
#define K_SLOCK 71
#define K_PAUSE 72
#define K_INSERT 73
#define K_HOME 74

#define K_PGUP 75
#define K_DELETE 76
#define K_END 77
#define K_PGDN 78
#define K_RIGHT 79

#define K_LEFT 80
#define K_DOWN 81
#define K_UP 82
#define K_NLOCK 83
#define K_KP_FSLASH 84

#define K_KP_ASTRIX 85
#define K_KP_MINUS 86
#define K_KP_PLUS 87
#define K_KP_ENTER 88
#define K_KP_1 89

#define K_KP_2 90
#define K_KP_3 91
#define K_KP_4 92
#define K_KP_5 93
#define K_KP_6 94

#define K_KP_7 95
#define K_KP_8 96
#define K_KP_9 97
#define K_KP_0 98
#define K_KP_DOT 99

#define K_100 100
#define K_APP 101
#define K_POWER 102
#define K_KP_EQL 103
#define K_F13 104

#define K_F14 105
#define K_F15 106
#define K_F16 107
#define K_F17 108
#define K_F18 109

#define K_F19 110
#define K_F20 111
#define K_F21 112
#define K_F22 113
#define K_F23 114

#define K_F24 115
#define K_EXCT 116
#define K_HELP 117
#define K_MENU 118
#define K_SELECT 119

#define K_SHIFT 225
#define K_CANCEL 155


int piezoPin = 13;
const  byte ROWS = 13; //changed library to suit
const  byte COLS = 11; //changed library to suit

// define USB scan codes to be send on each keystroke
// can use modifiers by adding them to the key number
// this can be used to mimic the key presses that
// don't match regular keyboads

 int stdKeys[ROWS][COLS] = {
  {K_F6, 0, 0, K_UP, 0, K_DOWN, K_B, K_H, K_N, K_T, K_Z},
  {0, 0, 0, K_HOME, K_LEFT, K_END, K_A, K_G, K_M, K_S, K_Y},
  {0, K_F8, 0, 0, 0, 0,  K_SHIFT, K_F, K_L, K_R, K_X},
  {K_ESC, K_F4, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {K_ESC, K_F3, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {K_ESC, K_F2, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, K_F1, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, K_PGUP, K_RIGHT, K_PGDN, K_C, K_I, K_O, K_U, K_MNS},
  {0, 0, 0, 0, 0, 0, K_D, K_J, K_P, K_V, K_9 + _S},
  {K_INSERT, 0, 0, 0, 0, 0, K_E, K_K, K_Q, K_W, K_0 + _S},
  {K_TAB, 0, 0, 0, 0, 0, K_7, K_4, K_1, K_COMMA, K_CANCEL},
  {K_BSPC, 0, 0, 0, 0, 0, K_8, K_5, K_2, K_0, K_SPACE},
  {K_ESC, 0, 0, 0, 0, 0, K_9, K_6, K_3, K_PERIOD, K_ENTER},
};

// define codes send when keys are in shifted mode
 int shiftKeys[ROWS][COLS] = {
  {63+ _S, 0, 0, 82+ _S, 0, 81+ _S, K_B+ _S, 11+ _S, 17+ _S, 23+ _S, 29+ _S},
  {0, 0, 0, 74+ _S, 80+ _S, 77+ _S, K_A+ _S, 10+ _S, 16+ _S, 22+ _S, 28+ _S},
  {0, 65+ _S, 0, 0, 0, 0,  K_SHIFT, 9, 15+ _S, 21+ _S, 27+ _S},
  {41+ _S, 61+ _S, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {41+ _S, 60+ _S, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {41+ _S, 59+ _S, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 58+ _S, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 75+ _S, 79+ _S, 78+ _S, 6+ _S, 12+ _S, 18+ _S, 24+ _S, 45+ _S},
  {0, 0, 0, 0, 0, 0, 7+ _S, 13+ _S, 19+ _S, 25+ _S, K_LBRKT},
  {73+ _S, 0, 0, 0, 0, 0, 8, 14+ _S, 20+ _S, 26+ _S, K_RBRKT},
  {43+ _S, 0, 0, 0, 0, 0, 36+ _S, 33+ _S, 37+ _S, 54+ _S, K_CANCEL},
  {42+ _S, 0, 0, 0, 0, 0, 31+ _S, 33+ _S, 30+ _S, 39+ _S, K_SPACE+ _S },
  {41+ _S, 0, 0, 0, 0, 0, 38+ _S, 35+ _S, 32+ _S, 55+ _S, K_ENTER+ _S },
};

// define codes sent if zero is found in either of
// the above arrays.
// used to assign weird keypresses for non keyboard functions
 
 int altKeys[ROWS][COLS] = {
  {0, 7, 7, 0, 7, 0, 0, 0, 0, 0, 0},
  {7, 7, 8, 0, 0, 0, 0, 0, 0, 0, 0},
  {7, 0, 9, 7, 7, 7, 0, 0, 0, 0, 0},
  {0, 0, 10, 7, 7, 20, 7, 4, 7, 7, 7},
  {0, 0, 11, 7, 7, K_PGDN, 7, 5, 7, 7, 7},
  {0, 0, 12, 7, 7, 22, 7, K_0, 7, 7, 7},
  {7, 0, 13, 7, 0/*chip stop */, 23, 7, 7, 7, 7, 7},
  {7, 7, 14, 0, 0, 0, 0, 0, 0, 0, 0},
  {7, 7, 15, 7, 7, 24, 0, 0, 0, 0, 0},
  {0, 7, 16, 7, 7, 25, 0, 0, 0, 0, 0},
  {0, 7, 17, 7, 7, 26, 0, 0, 0, 0, 0},
  {0, 7, 18, 7, 7, 27, 0, 0, 0, 0, 0},
  {0, 7, 19, 7, 7, 28, 0, 0, 0, 0, 0},
};

uint8_t buf[8] = { 0 }; //Keyboard report buffer
int shifted = 0;
int usbCode = 0;
int altered = 0;

 byte rowPins[ROWS] = {47, 45, 43, 41, 39, 37, 35, 33, 31, 29, 27, 25, 23}; //connect to the row pinouts of the keypad
 byte colPins[COLS] = {12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2}; //connect to the column pinouts of the keypad

Keypad stdpad = Keypad( makeKeymap(stdKeys), rowPins, colPins, ROWS, COLS );

void releaseKey() {
  buf[0] = 0;
  buf[2] = 0;
  Serial.write(buf, 8); // Send Release key
}

void setup() {
  Serial.begin(9600);
}

void loop() {

  // Fills stdpad.key[ ] array with up-to 10 active keys.
  // Returns true if there are ANY active keys.
  if (stdpad.getKeys())
  {
    for (int i = 0; i < LIST_MAX; i++) // Scan the whole key list.
    {
      if ( stdpad.key[i].stateChanged )   // Only find keys that have changed state.
      {
        // get usb key code from the standard or shifted array
        altered = 0;
        if (shifted) { usbCode = (( int*)shiftKeys)[stdpad.key[i].kcode];
        } else { usbCode = stdpad.key[i].kint; }
        // if still zero check the alternative aray
        if (usbCode == 0) {
          usbCode = (( int*)altKeys)[stdpad.key[i].kcode];
          altered = 1;
        }
        
        // always use the standard key array for state
        switch (stdpad.key[i].kstate) {  // Report active key state : IDLE, PRESSED, HOLD, or RELEASED
          case PRESSED:
                if (usbCode == K_ENTER) { // enter
                  tone(piezoPin, 2000, 200);
                } else if (usbCode == K_ESC) {  // escape
                  tone(piezoPin, 700, 200);
                } else if (usbCode == K_SHIFT) { // shift
                  tone(piezoPin, 1500, 100);
                } else if ( altered ){
                  tone(piezoPin, 500, 100); 
                } else {
                  tone(piezoPin, 1000, 100);
                }
            // Haas needs some keys faked - it doesn't
            // exactly follow normal keyboard shifted 
            // patterns
            if (usbCode >= 256 && shifted == 0) {
                buf[0] = KEY_LEFT_SHIFT;
                buf[2] = (usbCode & 255);
                Serial.write(buf, 8); // Send keypress
                break;
            // shift is not reported to USB
            } else if( usbCode == K_SHIFT ) {
               shifted = 1;
               break;
            // set the shift modifier code; send USB scan code
            } else if ( shifted ) {
               buf[0] = (usbCode >> 8);
               buf[2] = (usbCode & 255);
               Serial.write(buf, 8); // Send keypress
               break;
            // no modifier code; send USM scan code
            } else {
               buf[0] = 0;
               buf[2] = usbCode;
               Serial.write(buf, 8); // Send keypress
               break;
            }

          case HOLD:
            if (usbCode == K_SHIFT) { //shift
                buf[0] = 0;
                buf[2] = 57; //capslock
                Serial.write(buf, 8); // Send keypress
                tone(piezoPin, 2500, 100);
            }
 
            break;
          case RELEASED:
            switch ( usbCode ) {
              case K_SHIFT:
                shifted = 0;
                break;
              default:
                releaseKey();
                break;
            }
        }
      }
    }
  }
  }  // End loop
